'*Header-file for MEGSV4w32.DLL'
'Copyright ME-Messysteme GmbH H.Kabelitz*'

Attribute VB_Name = "ME_GSV4_API_W32"
Option Explicit
Public Const GSV_OK As Long = 0
Public Const GSV_ERROR As Long = -1
Public Const GSV_TRUE As Long = 1

Public Const OWN_ERR_MASK As Long = &H20000000
Public Const ERR_MUTEXFAILED As Long = &H300000F0
Public Const ERR_EVENTFAILED As Long = &H300000F1
Public Const ERR_PAR_NO As Long = &H300000F2
Public Const ERR_MEM_ALLOC	As Long = &H300000F3
Public Const ERR_NO_GSV4_FOUND As Long =  &H300000F4
Public Const ERR_BYTES_WRITTEN As Long = &H300000F5
Public Const ERR_WRONG_PARAMETER As Long = &H30000100
Public Const ERR_NO_GSV_ANSWER As Long = &H30000058
Public Const ERR_WRONG_ANSWER_NUM As Long = &H30000059
Public Const ERR_WRONG_ANSWER As Long = &H30000060
Public Const ERR_WRONG_FRAME_SUFFIX As Long = &H30000061
Public Const ERR_NOT_SUPPORTED As Long = &H30000062
Public Const ERR_WRONG_COMNO As Long = &H30000101
Public Const ERR_COM_ALREADY_OPEN As Long = &H300000F6
Public Const ERR_COM_GEN_FAILURE &H3000001F

Public Const ACTEX_FLAG_HANDSHAKE	As Long = &H04&
Public Const ACTEX_FLAG_WAIT_EXTENDED	As Long = &H00000100
Public Const ACTEX_FLAG_STOP_TX	As Long = &H00000200

Public Declare Function GSV4dllVersion Lib "MEGSV4w32" Alias "#39" () As Long

'Opens a GSV4 on COM-port specified by parameters, with all necessary ressources. Returns GSV_OK,if successful or GSV_ERROR'
Public Declare Function GSV4actExt Lib "MEGSV4w32" Alias "#1" (ByVal ComNo As Long, ByVal Bitrate As Long, ByVal BufSize As Long) As Long

'Same as above, but the following flags can be used:'
'ACTEX_FLAG_HANDSHAKE: if set, opens the port with hardware-handshake (rts-cts)'
'ACTEX_FLAG_WAIT_EXTENDED: 	//waits longer for device-answer in GSV4activateExtended'
'ACTEX_FLAG_STOP_TX	0x0200	//stops continious data transmission in GSV4activateExtended '
Public Declare Function GSV4activateExtended Lib "MEGSV4w32" Alias "#36" (ByVal ComNo As Long, ByVal Bitrate As Long, ByVal BufSize As Long, ByVal flags As Long) As Long

'Returns System Error Codes (http://msdn.microsoft.com/en-us/library/ms681381%28VS.85%29.aspx) retrieved by GetLastError(),'
'which was called by previously failed function or returns above defined errorcodes.'
'Can be called after another function returned GSV_ERROR. No GSV-Access'
Public Declare Function GSV4getLastError Lib "MEGSV4w32" Alias "#2" (ByVal ComNo As Long) As Long

'Clears Dll-buffers. Returns GSV_OK or GSV_ERROR. No GSV-access'
Public Declare Function GSV4clearDLLbuffer Lib "MEGSV4w32" Alias "#3" (ByVal ComNo As Long) As Long

'return-value: Number of measuring-values in buffer. No GSV-access'
'Chan=0: lowest number of all 4 buffers Chan=5: Highest buffer filling'
Public Declare Function GSV4received Lib "MEGSV4w32" Alias "#4" (ByVal ComNo As Long, ByVal Chan As Long) As Long

'Reads measuring values of all 4 channels. Returns GSV_OK, if at least one buffer is emty, GSV_TRUE, if values were read or GSV_ERROR.'
'Values are written where <out0>..<out3> point to, if GSV4readAll returns GSV_TRUE. No GSV-Access'
Public Declare Function GSV4readAll Lib "MEGSV4w32" Alias "#5" (ByVal ComNo As Long, ByRef out0 As Double, ByRef out1 As Double, ByRef out2 As Double, ByRef out3 As Double) As Long

'Reads one value of channel specified by <Chan> (within 1..4) where <out> points to,'
'if GSV4read returns GSV_TRUE. Otherwise GSV_OK, if the buffer is emty,'
'or GSV_ERROR. Chan must be within 1 to 4. No GSV-access'
Public Declare Function GSV4read Lib "MEGSV4w32" Alias "#6" (ByVal ComNo As Long, ByVal Chan As Long, ByRef out As Double) As Long

'Reads whole Array of values in dll-value-buffer(s), if returned GSV_TRUE. No GSV-Access.'
'Reads up to <count> values and writes number of values, which were really read, into <valsread>. <count> must be >0.'
'If resulting <valsread> is smaller than <count>, the whole buffer (the smallest, if Chan=0) was read.'
'If Chan=0: Reads whole Array of all channels into the array of double-type, where <out> points to. <count> must be dividible by 4.'
'<out> is sorted as follows: Chan1 in n*4, Chan2 in (n*4)+1, Chan3 in (n*4)+2, Chan4 in (n*4)+3 with n= {0..(<valsread> /4)-1}'
'E.g. if <valsread>=8, the content of <out> is, beginning with index=0: oldCh1,oldCh2,oldCh3,oldCh4,newCh1,newCh2,newCh3,newCh4'
'Returns GSV_OK, if at least one buffer is emty, GSV_TRUE, if values were read or GSV_ERROR.'
'If Chan={1..4}: Reads whole Array of specified channel. Oldest value is at index 0 (base) of <out>-array, newest at <valsread>-1.'
'Returns GSV_OK, if the buffer is emty, GSV_TRUE, if value(s) were read or GSV_ERROR.'
Public Declare Function GSV4readMultiple Lib "MEGSV4w32" Alias "#7" (ByVal ComNo As Long, ByVal Chan As Long, ByRef out As Double, ByVal count As Long, ByRef valsread As Long) As Long

'starts GSV-4 value transmission. State not saved in device.'
Public Declare Function GSV4startTX Lib "MEGSV4w32" Alias "#8" (ByVal ComNo As Long) As Long

'stops GSV-4 value transmission. State not saved in device.'
Public Declare Function GSV4stopTX Lib "MEGSV4w32" Alias "#9" (ByVal ComNo As Long) As Long

'Must be used for Cleaning dll-ressources at program-end. No GSV-Access'
Public Declare Function GSV4release Lib "MEGSV4w32" Alias "#10" (ByVal ComNo As Long) As Long

'Tares specified Channel to 0. If Chan is =0: tares all 4 Channels, otherwise: Chan: 1..4 '
Public Declare Function GSV4setZero Lib "MEGSV4w32" Alias "#11" (ByVal ComNo As Long, ByVal Chan As Long) As Long

'Store GSV-4-parameters at DataNo 2 or 3. '
Public Declare Function GSV4saveAll Lib "MEGSV4w32" Alias "#12" (ByVal ComNo As Long, ByVal DataNo As Long) As Long

'Loads GSV-4-parameters from DataNo 2 or 3 or Default-Settings from DataNo =1.  '
Public Declare Function GSV4getAll Lib "MEGSV4w32" Alias "#13" (ByVal ComNo As Long, ByVal DataNo As Long) As Long

'Change datarate with this. If desired datarate in Freq not available, it sets next possible value. '
Public Declare Function GSV4writeFreq Lib "MEGSV4w32" Alias "#14" (ByVal ComNo As Long, ByVal Freq As Double) As Long

'Read datarate with this.'
Public Declare Function GSV4readFreq Lib "MEGSV4w32" Alias "#15" (ByVal ComNo As Long) As Double

'Read Serialnumber with this. After called, serialnumber is in location where <SerNo> points to.'
'Public Declare Function int GSV4readSerialNo(int ComNo, char* SerNo);'
Private Declare Function MeGSV4readSerialNo Lib "MEGSV4w32" Alias "#17" (ByVal ComNo As Long, ByVal number As String) As Long

Public Declare Function GSV4writeThreshold Lib "MEGSV4w32" Alias "#18" (ByVal ComNo As Long, ByVal ThresNo As Long, ByVal ThresOn As Double, ByVal ThresOff As Double) As Long

Public Declare Function GSV4readThreshold Lib "MEGSV4w32" Alias "#19" (ByVal ComNo As Long, ByVal ThresNo As Long, ByRef ThresOn As Double, ByRef ThresOff As Double) As Long

Public Declare Function GSV4setMode Lib "MEGSV4w32" Alias "#20" (ByVal ComNo As Long, ByVal Mode As long, ByVal pw As String) As Long

Public Declare Function GSV4getMode Lib "MEGSV4w32" Alias "#21" (ByVal ComNo As Long) As Long

'Read Firmwareversion with this.'
Public Declare Function GSV4firmwareVersion Lib "MEGSV4w32" Alias "#22" (ByVal ComNo As Long) As Long

' Use this to trigger single transmission of one measuring value (all 4 channels).'
'Mainly used, if GSV-4 value transmission is switched off by GSV4stopTX.'
'After called, retrieve that value with GSV4read or GSV4readAll.'
Public Declare Function GSV4getValue Lib "MEGSV4w32" Alias "#23" (ByVal ComNo As Long) As Long

' Changes Baudrate of the GSV-4's serial interface.'
'Available Baudrates are:'
   '9600,'
   '19200,'
   '38400,'
   '57600,'
   '115200,'
   '230400 '
Public Declare Function GSV4setBaudSerial Lib "MEGSV4w32" Alias "#26" (ByVal ComNo As Long, ByVal Bitrate As Long) As Long

' Reads Baudrate of the GSV-4's serial interface.'
Public Declare Function GSV4getBaudSerial Lib "MEGSV4w32" Alias "#27" (ByVal ComNo As Long) As Long

'Change properties of input specified by <Chan>.
'<Gain> can be one of the following:'
'0: Channel is off.'
'1: Input is set to bridge-sensor, Sensitivity is 2 mV/V.'
'2: Input is set to bridge-sensor, Sensitivity is 10 mV/V.'
'3: Input is set to voltage-input, Sensitivity is +5V (0 to 5V).'
'4: Input is set to temperature-sensor of type PT1000.'
'5: Input is set to bridge-sensor, Sensitivity is 0.1 mV/V.'
Public Declare Function GSV4setGain Lib "MEGSV4w32" Alias "#28" (ByVal ComNo As Long, ByVal Chan As Long, ByVal gain As Long) As Long

'Read properties of input specified by <Chan>. See coding above.'
Public Declare Function GSV4getGain Lib "MEGSV4w32" Alias "#29" (ByVal ComNo As Long, ByVal Chan As Long) As Long

'Change properties of input specified by <Chan>.'
'<type> can be one of above defined enum-type "AinType":'
'AinTypeBridge2 : Input is set to bridge-sensor, Sensitivity is 2 mV/V.'
'AinTypeBridge10 : Input is set to bridge-sensor, Sensitivity is 10 mV/V.'
'AinTypeVolt5 : Input is set to voltage-input, Sensitivity is +5V (0 to 5V).'
'AinTypePT1000 : Input is set to temperature-sensor of type PT1000.'
'AinType_K=6 : Input is set to temperature-sensor of type K-cable'
'AinTypeVolt10 : Input is set to voltage-input, Sensitivity is +10V (0 to 10V).'
Public Declare Function GSV4setAnalogueType Lib "MEGSV4w32" Alias "#48" (ByVal ComNo As Long, ByVal Chan As Long, ByVal type As Long) As Long

'Reads properties of all 4 channel's inputs. If the return-value is GSV_OK, the input types are written to,
'where typCh1..typCh4 point to.'
'typCh1..typCh4 can be one of above defined enumerated type AinType. Example of usage:'
Public Declare Function GSV4getAnalogueType Lib "MEGSV4w32" Alias "#49" (ByVal ComNo As Long, ByRef typCh1 As Long, ByRef typCh2 As Long, ByRef typCh3 As Long, ByRef typCh4 As Long) As Long


'Write Display-scaling to GSV-4. This value is not used by the device, but will remain stored in it.'
'Normally used by application-software for scaling the measuring-values by multiplying them with the value'
'stored previously by GSV4DispSetNorm and read by GSV4DispGetNorm.'
Public Declare Function GSV4DispSetNorm Lib "MEGSV4w32" Alias "#30" (ByVal ComNo As Long, ByVal Chan As Long, ByVal Norm As Double) As Long

'Read Display-scaling stored in GSV-4. This value is not used by the device, but will remain stored in it.'
'Normally used by application-software for scaling the measuring-values by multiplying them with the value'
'stored previously by GSV4DispSetNorm and read by GSV4DispGetNorm.'
Public Declare Function GSV4DispGetNorm Lib "MEGSV4w32" Alias "#31" (ByVal ComNo As Long, ByVal Chan As Long) As Double

' Changes Baudrate of the GSV-4's CAN interface.'
'Available Baudrates are in kBis/s:'
   '20,   50,   80,   100,   125,   250,   500,   1000 '
Public Declare Function GSV4setBaudCAN Lib "MEGSV4w32" Alias "#32" (ByVal ComNo As Long, ByVal Bitrate As Long) As Long

' Reads Baudrate of the GSV-4's CAN interface.'
Public Declare Function GSV4getBaudCAN Lib "MEGSV4w32" Alias "#33" (ByVal ComNo As Long) As Long

' Changes ID of the GSV-4's CAN interface. Each device has 4 different IDs, which are specified by <IDno>'
'and can be one of the following:'
'TX_1:	1 TX_2:	2 RX_1:	5 RX_2:	6 '
Public Declare Function GSV4setCanID Lib "MEGSV4w32" Alias "#34" (ByVal ComNo As Long, ByVal IDno As Long, ByVal CanID As Long) As Long

' Reads ID of the GSV-4's CAN interface. Each device has 4 different IDs, which are specified by <IDno>'
Public Declare Function GSV4getCanID Lib "MEGSV4w32" Alias "#35" (ByVal ComNo As Long, ByVal IDno As Long) As Long

' Sets / reads unit-Number '
Public Declare Function GSV4setUnit Lib "MEGSV4w32" Alias "#40" (ByVal ComNo As Long, ByVal Chan As Long, ByVal UnitNo As Long) As Long

Public Declare Function GSV4getUnit Lib "MEGSV4w32" Alias "#41" (ByVal ComNo As Long, ByVal Chan As Long) As Long

'Public Declare Function int GSV4setUnitText(int ComNo, int Chan, char* UnitTxt);'

'Public Declare Function int GSV4getUnitText(int ComNo, int Chan, char* UnitTxt);'

'Public Declare Function int GSV4setDigitalIOtype(int ComNo, int DigIO_No, DigIOtype type);	//DigIO_No=0: all Bits, else: 1..8'

'Public Declare Function int GSV4getDigitalIOtype(int ComNo, int DigIO_No);	//DigIO_No=0: all Bits, else: 1..8'

'Public Declare Function int GSV4setDigitalIOvalue(int ComNo, int DigIO_No, int value);	//DigIO_No=0: all Bits, else: 1..8'

'Public Declare Function long GSV4getDigitalIOvalue(int ComNo, int DigIO_No);	//DigIO_No=0: all Bits, else: 1..8'

'Val=1: Permanent data transmission = on'
Public Declare Function GSV4setTXmode Lib "MEGSV4w32" Alias "#50" (ByVal ComNo As Long, ByVal val As Long) As Long

Public Declare Function GSV4getTXmode Lib "MEGSV4w32" Alias "#51" (ByVal ComNo As Long) As Long

'Public Declare Function int GSV4writeOffset(int ComNo, int Chan, int gain, long offset);'

'Public Declare Function long GSV4readOffset(int ComNo, int Chan, int gain);'

'Public Declare Function int GSV4writeScaledOffset(int ComNo, int Chan, int gain, double offset);'

'Public Declare Function double GSV4readScaledOffset(int ComNo, int Chan, int gain);'

'Public Declare Function int MeGSV4setScale(int ComNo, int Chan, int gain, long Scale);'

'Public Declare Function long MeGSV4getScale(int ComNo, int Chan, int gain);'

'Public Declare Function int MeGSV4writeSerialNo(int ComNo, char* SerNo);'

Public Function GSV4readSerialNo(ByVal ComNo As Long, ByRef SerNo As String) As Long
    Dim sernum As String, rcode As Long, cnum As Long
    sernum = String(10, vbNullChar)
    rcode = MeGSV4readSerialNo(ComNo, sernum)
    cnum = InStr(1, sernum, vbNullChar)
    If rcode = GSV_OK Then
        If cnum > 0 Then sernum = Left(sernum, cnum - 1)
        SerNo = sernum
    End If
    GSV4readSerialNo = rcode
End Function